#!/usr/bin/env python
# -*- coding: utf-8 -*-

import uuid
import os


def newUUID():
    return str(uuid.uuid1())


class Endpoints():
    DEFAULT_HOST = "https://engine-aiearth.aliyun.com"
    HOST = os.getenv("SDK_CLIENT_HOST", DEFAULT_HOST)

    MAP = HOST + "/ep/v1alpha/map"
    MAP_VECTOR = HOST + "/ep/v1alpha/vector"
    SDK_GATEWAY = HOST + "/ep/v1alpha/api"
    INTERACTIVE_SESSION = HOST + "/ep/v1alpha/interactive_session"
    AUTH_PORTAL_PAGE = HOST + "/#/utility/auth-token"

    MARIANA_DATA = "http://dataset-upload.engine-aiearth.aliyun.com"

    ML_PROXY_HOST = HOST + "/ep/v1alpha/mlproxy/"

    DEFAULT_STAC_ENDPOINT = "http://172.16.2.12:8038"
    STAC_ENDPOINT = os.getenv("STAC_ENDPOINT", DEFAULT_STAC_ENDPOINT)

    DEFAULT_OPENAPI_ENDPOINT = "aiearth-engine.cn-hangzhou.aliyuncs.com"
    OPENAPI_ENDPOINT = os.getenv("OPENAPI_ENDPOINT", DEFAULT_OPENAPI_ENDPOINT)

    DEFAULT_OPENAPI_REGION_ID = "cn-hangzhou"
    OPENAPI_REGION_ID = os.getenv("OPENAPI_REGION_ID", DEFAULT_OPENAPI_REGION_ID)


class SdkGatewayResource():
    class Task():
        START = "/task/start"
        STATUS = "/task/status/{}"
        CANCEL = "/task/cancel/{}"

    class Maps():
        GET_MAP_ID = "/maps/getMapId"


class InteractiveSessionResource():
    CREATE = "/job/session/create"
    QUERY = "/job/session/query?session_id={}"
    EXEC = "/job/session/script/exec"
    CANCEL_ALL = "/job/session/script/cancelAll"


class MarianaDataResource():
    GET_STS_TOKEN = "/dataapi/public/oss/sts/token/get?suffix={}&type=DATASET"
    PUBLISH = "/dataapi/public/dataset/publish"
    GET_PUBLISH_STATUS = "/dataapi/public/dataset/status?id={}"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from . import g_var
from .client import Endpoints
from .error import AIEError, AIEErrorCode
from datetime import datetime


class Authenticate(object):
    class ClientId(object):
        USER_STD = "user_std"
        ALIYUN_JUPYTER = "aliyun_jupyter"

    @staticmethod
    def getClientEnvironment():
        if os.getenv(g_var.JupyterEnv.TOKEN) is not None:
            return Authenticate.ClientId.ALIYUN_JUPYTER
        else:
            return Authenticate.ClientId.USER_STD

    @staticmethod
    def __displayAuthPromt():
        print("请将以下地址粘贴到Web浏览器中，访问授权页面，并将个人token粘贴到输入框中")
        print("\t", Endpoints.AUTH_PORTAL_PAGE)

    @staticmethod
    def __getTokenFromJupyter():
        token = os.getenv(g_var.JupyterEnv.TOKEN)
        if token is None:
            raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                           "云平台环境token获取失败")
        return token

    @staticmethod
    def auth(token=None, access_key_id=None, access_key_secret=None):
        client_id = Authenticate.getClientEnvironment()

        cred = {}
        cred["client_id"] = client_id
        if token is not None:
            cred["token"] = token
        elif access_key_id and access_key_secret:
            token = Authenticate.__get_token_by(access_key_id, access_key_secret)
            cred["token"] = token
        else:
            if client_id == Authenticate.ClientId.ALIYUN_JUPYTER:
                token = Authenticate.__getTokenFromJupyter()
                cred["token"] = token
            else:
                Authenticate.__displayAuthPromt()
                token = input("个人token: ")
                cred["token"] = token

        g_var.set_var(g_var.GVarKey.Authenticate.CLIENT_ID, cred["client_id"])
        g_var.set_var(g_var.GVarKey.Authenticate.TOKEN, cred["token"])

    @staticmethod
    def getCurrentUserToken():
        if not g_var.has_var(g_var.GVarKey.Authenticate.CLIENT_ID):
            raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                           "客户端ID获取失败，请先调用 aie.Authenticate() 进行授权")

        client_id = g_var.get_var(g_var.GVarKey.Authenticate.CLIENT_ID)
        token = ""
        if client_id == Authenticate.ClientId.ALIYUN_JUPYTER:
            token = Authenticate.__getTokenFromJupyter()
        else:
            if not g_var.has_var(g_var.GVarKey.Authenticate.TOKEN):
                raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                               "个人token获取失败，请先调用 aie.Authenticate() 进行授权")
            token = g_var.get_var(g_var.GVarKey.Authenticate.TOKEN)
        return token

    @staticmethod
    def __get_token_by(access_key_id, access_key_secret, force_create=False, expected_hours_before_expired=24):
        from alibabacloud_tea_openapi import models
        from alibabacloud_aiearth_engine20220609.client import Client
        from alibabacloud_aiearth_engine20220609.models import GetUserTokenRequest
        from alibabacloud_aiearth_engine20220609.models import GetUserTokenResponse
        config = models.Config(
            access_key_id=access_key_id, access_key_secret=access_key_secret,
            region_id=Endpoints.OPENAPI_REGION_ID, endpoint=Endpoints.OPENAPI_ENDPOINT)

        client = Client(config)

        get_user_token = GetUserTokenRequest()
        get_user_token.force_create = force_create
        get_user_token_resp: GetUserTokenResponse = client.get_user_token(get_user_token)
        token = get_user_token_resp.body.token
        expired_at_ts = get_user_token_resp.body.expired_at / 1000.

        current_ts = datetime.now().timestamp()
        if expired_at_ts - current_ts < expected_hours_before_expired * 3600.:
            get_user_token = GetUserTokenRequest()
            get_user_token.force_create = True
            get_user_token_resp: GetUserTokenResponse = client.get_user_token(get_user_token)
            return get_user_token_resp.body.token
        else:
            return token



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os

import requests
from ..error import AIEError, AIEErrorCode
from .. import g_var
from .. import auth
from .. import env

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class BaseClient(object):
    @staticmethod
    def __append_extra_hdrs(hdrs):
        hdrs["x-aie-auth-token"] = auth.Authenticate.getCurrentUserToken()

        project_id = os.getenv(g_var.JupyterEnv.PROJECT_ID, "local")
        hdrs['x-aie-client-name'] = f"aie-sdk@Project-{project_id}"
        return hdrs

    @staticmethod
    def post(url, hdrs, data, append_extra_hdrs=True):
        if append_extra_hdrs:
            hdrs = BaseClient.__append_extra_hdrs(hdrs)

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::post request. url: {url}, headers: {json.dumps(hdrs)}, data: {json.dumps(data)}")

        resp = requests.post(url=url, headers=hdrs, timeout=(600, 600),
                             json=data, verify=False)

        if resp.status_code != 200:
            if "401 Authorization Required" in resp.text:
                raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                               f"未授权或者个人token失效，请先调用 aie.Authenticate() 进行授权")
            else:
                raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                               "", f"http请求错误: {resp.text}")

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::post response. url: {url}, response: {resp.json()}")

        return resp

    @staticmethod
    def get(url, hdrs, append_extra_hdrs=True):
        if append_extra_hdrs:
            hdrs = BaseClient.__append_extra_hdrs(hdrs)

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::get request. url: {url}, headers: {json.dumps(hdrs)}")

        resp = requests.get(url=url, headers=hdrs,
                            timeout=(600, 600), verify=False)

        if resp.status_code != 200:
            if "401 Authorization Required" in resp.text:
                raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                               f"未授权或者个人token失效，请先调用 aie.Authenticate() 进行授权")
            else:
                raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                               "", f"http请求错误: {resp.text}")

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::get response. url: {url}, response: {resp.json()}")

        return resp

    @staticmethod
    def delete(url, hdrs, append_extra_hdrs=True):
        if append_extra_hdrs:
            hdrs = BaseClient.__append_extra_hdrs(hdrs)

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::delete request. url: {url}, headers: {json.dumps(hdrs)}")

        resp = requests.delete(url=url, headers=hdrs,
                               timeout=(600, 600), verify=False)

        if resp.status_code != 200:
            if "401 Authorization Required" in resp.text:
                raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                               f"未授权或者个人token失效，请先调用 aie.Authenticate() 进行授权")
            else:
                raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                               "", f"http请求错误: {resp.text}")

        if env.get_log_level() ==  g_var.LogLevel.DEBUG_LEVEL:
            print(
                f"BaseClient::delete response. url: {url}, response: {resp.json()}")

        return resp



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from .error import AIEError, AIEErrorCode

G_VAR = {}


class JupyterEnv(object):
    TOKEN = "JUPYTER_TOKEN"
    PROJECT_ID = "JUPYTERHUB_SERVER_NAME"


class LogLevel(object):
    DEBUG_LEVEL = "debug"
    INFO_LEVEL = "info"


class GVarKey(object):
    class Authenticate(object):
        CLIENT_ID = "client_id"
        TOKEN = "token"

    class InteractiveSession(object):
        INTERACTIVE_SESSION_ID = "interactive_session_id"

    class Project(object):
        PROJECT_ID = "project_id"

    class Log(object):
        LOG_LEVEL = "log_level"


def has_var(key):
    return key in G_VAR


def get_var(key):
    if key not in G_VAR:
        raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                       "", f"G_VAR {key} not found")
    return G_VAR[key]


def set_var(key, value):
    global G_VAR
    G_VAR[key] = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json


class AIEErrorCode(object):
    DEFAULT_INTERNAL_ERROR = 12306000

    # user define
    ENVIRONMENT_INIT_ERROR = 22306001
    ARGS_ERROR = 22306002
    JOB_CANCELLED = 22306003


class AIEError(Exception):
    DEFAULT_ERROR_MESSAGE = "系统未知异常"

    def __init__(self, code: int = AIEErrorCode.DEFAULT_INTERNAL_ERROR, message: str = "", troubleshooting_information: str = ""):
        debug_info = ""
        if troubleshooting_information != json.dumps({}):
            debug_info = troubleshooting_information
        if str(code).startswith("2"):
            super().__init__("[AIEError]: {}, {}. {}".format(
                code, message, debug_info))
        else:
            super().__init__("{}. {}".format(
                AIEError.DEFAULT_ERROR_MESSAGE, debug_info))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
from os.path import expanduser

DEFAULT_STAC_AIE_ML_COLLECTION = "AIE_PUBLIC_DATA_ML"

# keys of PAI Job params
PAI_JOB_PARAM_NAME = "name"
PAI_JOB_PARAM_PROJECT = "project"
PAI_JOB_PARAM_SCRIPT = "script"
PAI_JOB_PARAM_ENTRYFILE = "entryFile"
PAI_JOB_PARAM_INPUTS = "inputs"
PAI_JOB_PARAM_CHECKPOINT_DIR = "checkpointDir"
PAI_JOB_PARAM_WORKER_COUNT = "workerCount"
PAI_JOB_PARAM_OSS_HOST = "ossHost"
PAI_JOB_PARAM_PYTHON = "python"
PAI_JOB_PARAM_CLUSTER = "cluster"
PAI_JOB_PARAM_HYPER_PARAMETERS = "hyperParameters"

# local credentials
DEFAULT_CREDENTIALS_DIR = expanduser(Path.home().joinpath('.mlproxy', 'credentials'))
CREDENTIALS_FILE_NAME = "sts_info"
PAI_TMP_CREDENTIALS_FILE_NAME = ".tmp_info"


from . import g_var


def set_log_level(debug_level):
    g_var.set_var(g_var.GVarKey.Log.LOG_LEVEL, debug_level)


def get_log_level(default_level=g_var.LogLevel.INFO_LEVEL):
    if not g_var.has_var(g_var.GVarKey.Log.LOG_LEVEL):
        return default_level
    return g_var.get_var(g_var.GVarKey.Log.LOG_LEVEL)
